/*
 Copyright (c) 2004, KJK::Hyperion
 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this 
    list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the KJK::Namespace nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef IAM_RGROUP_H_INCLUDED_
#define IAM_RGROUP_H_INCLUDED_

#include "Items.h"
#include "Groups.h"

namespace IAM
{
 namespace RGroup
 {
  enum Region
  {
   Most,
   Others,
   SuperUsers,
   RegionCount
  };
 
  struct StepStateImpl:
   public std::binary_function<enum State, enum Transition, enum State>
  {
   StepStateImpl::result_type operator()
   (
    IN StepStateImpl::first_argument_type eState,
    IN StepStateImpl::second_argument_type eType
   )
   {
    switch(eType)
    {
     // "-" prefix
     case DoNegative:
     {
      switch(eState)
      {
       case Positive:
       case Neutral:
        return Neutral;
 
       case Negative:
       case Delete:
        assert(false);
      }
 
      break;
     }
 
     //* "+" prefix
     case DoPositive:
     {
      switch(eState)
      {
       case Neutral:
       case Positive:
        return Positive;
 
       case Negative:
       case Delete:
        assert(false);
      }
 
      break;
     }
 
     case DoDelete:
     case DoUndelete:
      assert(false);
    }
 
    return Neutral;
   } // operator()
  }; // struct StepStateImpl
 
  typedef ItemStateTransition StateTransition;
  typedef ItemState<StepStateImpl> State;
  typedef GroupItem Item;

  typedef ItemStates<Item, State, StateTransition> RegionStates;
 } // namespace RGroup
} // namespace IAM

#endif // IAM_GROUP_H_INCLUDED_

// EOF
